format short;


%  The Correlational Analysis of Dyad-Level Data in the Distinguishable-Case
%  'Gonzalez & Griffin, 1999, Personal Relationships, 6, 449-469.

%  the following command causes SPSS to read a data file where
%  the first collumn contains the dyad numbers (the dyad id# given
%  to both dyad members); the second collumn contains the person
%  id #s (e.g., sex) coded as 1 or 2; the third and subsequent
%  collumns contains the variables to be analyzed

%  the data files must have been sorted by dyad id# and by person id # 

%  there should be no missing values for any of the variables


% data from Griffin & Gonzalez (1995)
data = [
  4  1   52.5     20      5
  4  2   43.5     25      3
  6  1   48.5     64     17
  6  2   41.0     44      4
  7  1   33.5     50      2
  7  2   34.5     19      2
 12  1   45.5     38      8
 12  2   64.5     21     10
 14  1   25.0     26      0
 14  2   26.0     20      2
 17  1   10.5      4      1
 17  2   11.5      6      1
 19  1   51.0     35     12
 19  2   51.5     22      6
 21  1   55.0     27     20
 21  2   63.0     21      3
 22  1   36.5     10      3
 22  2   38.5     19      5
 23  1   59.0     27      4
 23  2   38.5     37      2
 27  1   48.0     29      0
 27  2   53.5     11     22
 29  1   45.5     25      6
 29  2   55.0     25     13
 30  1   40.0      6      3
 30  2   36.0      4      0
 31  1   27.0     76      4
 31  2   32.5     37     10
 33  1   44.0     29      5
 33  2   34.0     14      2
 34  1   45.5     64     45
 34  2   67.5     33     13
 35  1   30.0      9      0
 35  2   29.5     14      0
 37  1    0.0      0      0
 37  2    0.0      1      1
 40  1   31.0     13      0
 40  2   50.5     21      3
 42  1   41.5     26      2
 42  2   42.0     27     11
 43  1   33.5     20      9
 43  2   29.0     11      3
 45  1   62.0     25     22
 45  2   57.0     21      9
 47  1   51.0     23      1
 47  2   58.0     22      5
 48  1    1.5      2      0
 48  2    1.5      5      0  ];

% Done.  The program will can now analyze the data.

norig = size(data,1);
tailed = 2;

% selecting pairs that have the same dyad #s, and where sex=1 & sex=2;
datarev=repmat(-9999,1,size(data,2));
for aa = 1:size(data,1)-1;
if data(aa,1)==data(aa+1,1) & data(aa,2)==1 & data(aa+1,2)==2;
datarev = [ datarev ; data(aa:(aa+1) ,:) ];
end;
end;
data = datarev(2:size(datarev,1),2:size(data,2));

n = size(data,1) / 2;

% setting up pairwise ("double-counted") collumns of data;
xp = repmat(-9999,size(data,1),(size(data,2)-1));
for aa = 1:size(data,1)/2;
xp(aa*2-1,:) = data(aa*2  ,2:size(data,2));
xp(aa*2  ,:) = data(aa*2-1,2:size(data,2));
end;
m = [ data , xp ];

% correlation matrix;
nr = size(m,1);
rawsp = m' * m ;
rsums = sum(m,1)';
corsp = rawsp - (1/nr) * (rsums) * rsums' ;
vcv = corsp * (1/(nr-1));
d = inv(diag(sqrt(diag(vcv))));
cr = d * vcv * d;

% correlations between residuals -- Velicer, 1976;
c22 = cr(2:size(cr,1),2:size(cr,2));
c11 = cr(1,1);
c12 = cr(1,2:size(cr,2));
c22part = c22 - c12' * inv(c11) * c12 ;
d = diag(1 ./ (sqrt(diag(c22part))));
resd = d * c22part * d ;

overall = resd(1:(size(resd,1)/2), 1:(size(resd,2)/2));
iccross = resd(((size(resd,1)/2)+1):size(resd,1),1:(size(resd,1)/2));

% p-Levels of the intraclass correlations;
intra   = diag(iccross);
zintra  =  (sqrt(n) * intra) ./ (1 - intra.^2);
pzintra = (1 - normp(abs(zintra))) * 2;

% tests of assumptions involving variances & covariances;

% matrix of pairwise ("double-counted") collumns of data for just one sex;
half = repmat(-9999, 1, (size(m,2)-1) );
for aa = 1:size(m,1);
if (m(aa,1) == 1);
half = [ half; m(aa,(2:size(m,2)))  ];
end;
end;
half = half(2:size(half,1),:);

% variance-covariance matrix & correlation matrix;
nr2 = size(half,1);
rawsp2 = half' * half ;
rsums2 = sum(half,1)';
corsp2 = rawsp2 - (1/nr2) * (rsums2) * rsums2' ;
vcv2 = corsp2 * (1/(nr2-1));
d2 = inv(diag(sqrt(diag(vcv2))));
cr2 = d2 * vcv2 * d2;

first  = vcv2(1:(size(vcv2,1)/2), 1:(size(vcv2,2)/2));
second=vcv2((size(vcv2,1)/2+1):size(vcv2,1),(size(vcv2,2)/2+1):size(vcv2,2));
iccross2=cr2(((size(cr2,1)/2)+1):size(cr2,1),1:(size(cr2,1)/2));
iccrossv=vcv2(((size(vcv2,1)/2)+1):size(vcv2,1),1:(size(vcv2,1)/2));

% tests for equality of variances;
vartests = repmat(-9999,size(first,1), 6);
for bb = 1:size(first,1);
vartests(bb,1)=[ bb ];
vartests(bb,2:3)=[ first(bb,bb) ,second(bb,bb)  ];
vartests(bb,4)=((first(bb,bb)-second(bb,bb))*sqrt(size(half,1)-2)) ...
   / (2*sqrt(first(bb,bb)*second(bb,bb)*(1-(iccross2(bb,bb).^2))));
vartests(bb,5)=size(half,1)-2;
vartests(bb,6) = (1 - tp(abs(vartests(bb,4)),vartests(bb,5))) * 2;
end;

% tests for equality of the within-category population covariances;
covarswi = repmat(-9999,1 , 6);
for cc = 1:size(first,2)-1;
for rr = (cc+1):size(first,1);
line     = repmat(-9999,1 , 6);
line(1,1:2) = [ cc, rr ];
line(1,3:4) = [ first(cc,rr) , second(cc,rr) ];
line(1,5)=((first(cc,rr)-second(cc,rr))*sqrt(size(half,1))) / ...
  sqrt(first(cc,cc)*first(rr,rr)+second(cc,cc)*second(rr,rr)+ ...
  2*(((first(cc,rr)+second(cc,rr))/2)  .^2) - ...
  2*(iccrossv(cc,cc)*iccrossv(rr,rr)+iccrossv(cc,rr)*iccrossv(rr,cc)));
line(1,6) = (1 - normp(abs(line(1,5)))) * 2;
covarswi = [ covarswi ; line ];
end;
end;
covarswi = covarswi(2:size(covarswi,1),:);

% tests for equality of the across-category population covariances;
covarscr = repmat(-9999,1 , 6);
for cc = 1:size(first,2)-1;
for rr = (cc+1):size(first,1);
line     = repmat(-9999,1 , 6);
line(1,1:2) = [ cc, rr ];
line(1,3:4) = [ iccrossv(cc,rr) , iccrossv(rr,cc) ];
line(1,5)=((iccrossv(cc,rr)-iccrossv(rr,cc))*sqrt(size(half,1)))/ ...
  sqrt(first(cc,cc)*second(rr,rr)+second(cc,cc)*first(rr,rr)+ ...
  2*(((iccrossv(cc,rr)+iccrossv(rr,cc))/2)  .^2) - ...
  2*(first(cc,rr)*second(cc,rr)+iccrossv(cc,cc)*iccrossv(rr,rr)));
line(1,6) = (1 - normp(abs(line(1,5)))) * 2;
covarscr = [ covarscr ; line ];
end;
end;
covarscr = covarscr(2:size(covarscr,1),:);

% initializing;
nstarovr   = repmat(-9999,size(overall,1),size(overall,2));
zoverwi    = nstarovr;
pzoverwi   = nstarovr;
zovercr    = nstarovr;
pzovercr   = nstarovr;
indcorrs   = nstarovr;
tind       = nstarovr;
ptind      = nstarovr;
dyadcors   = nstarovr;
dstar      = nstarovr;
zdyad      = nstarovr;
pzdyad     = nstarovr;

for ii = 1:size(overall,1);
for jj = 1:size(overall,2);
if (ii ~= jj);

% p-Levels of the overall within-partner correlations;
nstarovr(ii,jj) = (2*n) / ...
       (1+iccross(ii,ii)*iccross(jj,jj)+iccross(ii,jj).^2);
zoverwi(ii,jj) = overall(ii,jj)*sqrt(nstarovr(ii,jj));
pzoverwi(ii,jj) = (1 - normp(abs(zoverwi(ii,jj)))) * 2;

% p-Levels of the overall cross-partner correlations;
zovercr(ii,jj) = iccross(ii,jj)*sqrt(nstarovr(ii,jj));
pzovercr(ii,jj) = (1 - normp(abs(zovercr(ii,jj)))) * 2;

% individual-level correlations;
indcorrs(ii,jj)=(overall(ii,jj)-iccross(ii,jj))/ ...
        (sqrt(1-iccross(ii,ii))*sqrt(1-iccross(jj,jj)));
if abs(indcorrs(ii,jj)) < 1;
tind(ii,jj) = (indcorrs(ii,jj)*sqrt(n-2)) / ...
        sqrt(1-(indcorrs(ii,jj).^2));
ptind(ii,jj) = (1 - tp(abs(tind(ii,jj)),(n-2))) * 2;
end

% dyad-level correlations;
if (iccross(ii,ii) > 0 & iccross(jj,jj) > 0);
dyadcors(ii,jj)=iccross(ii,jj)/ ...
        (sqrt(iccross(ii,ii))*sqrt(iccross(jj,jj)));
dstar(ii,jj)  = (2*n / (1+iccross(ii,ii)*iccross(jj,jj) ...
        +overall(ii,jj).^2))*iccross(ii,ii)*iccross(jj,jj);
zdyad(ii,jj)  = dyadcors(ii,jj)*sqrt(dstar(ii,jj));
pzdyad(ii,jj) = (1 - normp(abs(zdyad(ii,jj)))) * 2;
end;

end;
end;
end;

% preparing the output matrices;
owpart  = repmat(-9999,1,5);
ocpart  = owpart;
dyadrzp = owpart;
withrzp = repmat(-9999,1,6);
for cc = 1:size(overall,2)-1;
for rr = (cc+1):size(overall,1);
owpart=[ owpart ; cc, rr, overall(cc,rr),zoverwi(cc,rr), pzoverwi(cc,rr) ];
ocpart=[ ocpart ; cc, rr, iccross(cc,rr),zovercr(cc,rr), pzovercr(cc,rr) ];
dyadrzp=[ dyadrzp ; cc, rr, dyadcors(cc,rr),zdyad(cc,rr), pzdyad(cc,rr) ];
withrzp=[ withrzp ; cc, rr, indcorrs(cc,rr),tind(cc,rr), (n-2), ptind(cc,rr) ];
end;
end;
owpart  = owpart(2:size(owpart,1),:);
ocpart  = ocpart(2:size(ocpart,1),:);
dyadrzp = dyadrzp(2:size(dyadrzp,1),:);
withrzp = withrzp(2:size(withrzp,1),:);

disp([' ']);disp([' ']);
disp(['The Correlational Analysis of Dyad-Level Data']);disp([' ']);
disp(['in the Distinguishable-Case']);disp([' ']);
disp(['Gonzalez & Griffin, 1999, Personal Relationships, 6, 449-469.']);disp([' ']);
disp(['All Significance Tests Are Two-Tailed']);disp([' ']);
disp(['Number of Individual Cases in the Data File = ' num2str(norig) ]);disp([' ']);
disp(['Number of Individual Cases That Will be Processed = ' num2str(size(data,1)) ]);
disp([' ']);disp([' ']);disp(['Variance-Covariance Matrix for the Primary Variables:']);
disp([' ']);disp([vcv2]);disp([' ']);disp([' ']);
disp(['Tests for Equality of Variances:']);disp([' ']);
disp(['     Var.#  Variance  Variance       t       df          p']);
disp([vartests]);disp([' ']);disp([' ']);
disp(['Tests for Equality of the Within-Person Covariances:']);disp([' ']);
disp(['    Var.#     Var.#    Covar.    Covar.       z         p']);
disp([covarswi]);disp([' ']);disp([' ']);
disp(['Overall Within-Partner Correlations, z-values & p-Levels:']);disp([' ']);
disp(['     Var.#     Var.#       r        z         p']);
disp([owpart]);disp([' ']);disp([' ']);
disp(['Tests for Equality of the Across-Person Covariances:']);disp([' ']);
disp(['    Var.#     Var.#    Covar.    Covar.       z         p']);
disp([ covarscr]);disp([' ']);disp([' ']);  
disp(['Overall Cross-Partner Correlations, z-values & p-Levels:']);disp([' ']);
disp(['     Var.#     Var.#       r        z         p']);
disp([ocpart]);disp([' ']);disp([' ']); 
disp(['Intraclass Correlations, z-values & p-Levels:']);disp([' ']);
disp(['     Var.#       r        z            p']);
disp([[ (1:size(intra,1))', intra, zintra, pzintra] ]);disp([' ']);disp([' ']);
disp(['Dyad-Level Correlations, z-values & p-Levels:']);disp([' ']);
disp(['     Var.#     Var.#       r        z         p']);
disp([dyadrzp]);disp([' ']);disp([' ']); 
disp(['Individual-Level Correlations, t-values & p-Levels:']);disp([' ']);
disp(['     Var.#     Var.#       r        t        df         p']);
disp([withrzp]);disp([' ']);disp([' ']);
disp(['-9999 indicates that a value could not be computed.']);disp([' ']);
disp(['The computations sometimes produce dyad-level']);
disp(['correlations > 1.00 or < -1.00.  This is most likely to']);
disp(['occur when an intraclass correlations is weak.']); 
disp(['See Griffin & Gonzalez, 1995, p. 434-437 for details.']);


